#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/21 17:00
Desc: 东方财富网-数据中心-特色数据-股票账户统计
东方财富网-数据中心-特色数据-股票账户统计: 股票账户统计详细数据
https://data.eastmoney.com/cjsj/gpkhsj.html
"""

import pandas as pd
import requests


def stock_account_statistics_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股票账户统计
    https://data.eastmoney.com/cjsj/gpkhsj.html
    :return: 股票账户统计数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_STOCK_OPEN_DATA",
        "columns": "ALL",
        "pageSize": "500",
        "sortColumns": "STATISTICS_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "pageNumber": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "数据日期",
        "新增投资者-数量",
        "新增投资者-环比",
        "新增投资者-同比",
        "期末投资者-总量",
        "期末投资者-A股账户",
        "期末投资者-B股账户",
        "上证指数-收盘",
        "上证指数-涨跌幅",
        "沪深总市值",
        "沪深户均市值",
        "-",
    ]
    temp_df = temp_df[
        [
            "数据日期",
            "新增投资者-数量",
            "新增投资者-环比",
            "新增投资者-同比",
            "期末投资者-总量",
            "期末投资者-A股账户",
            "期末投资者-B股账户",
            "沪深总市值",
            "沪深户均市值",
            "上证指数-收盘",
            "上证指数-涨跌幅",
        ]
    ]
    temp_df["新增投资者-数量"] = pd.to_numeric(
        temp_df["新增投资者-数量"], errors="coerce"
    )
    temp_df["新增投资者-环比"] = pd.to_numeric(
        temp_df["新增投资者-环比"], errors="coerce"
    )
    temp_df["新增投资者-同比"] = pd.to_numeric(
        temp_df["新增投资者-同比"], errors="coerce"
    )
    temp_df["期末投资者-总量"] = pd.to_numeric(
        temp_df["期末投资者-总量"], errors="coerce"
    )
    temp_df["期末投资者-A股账户"] = pd.to_numeric(
        temp_df["期末投资者-A股账户"], errors="coerce"
    )
    temp_df["期末投资者-B股账户"] = pd.to_numeric(
        temp_df["期末投资者-B股账户"], errors="coerce"
    )
    temp_df["沪深总市值"] = pd.to_numeric(temp_df["沪深总市值"], errors="coerce")
    temp_df["沪深户均市值"] = pd.to_numeric(temp_df["沪深户均市值"], errors="coerce")
    temp_df["上证指数-收盘"] = pd.to_numeric(temp_df["上证指数-收盘"], errors="coerce")
    temp_df["上证指数-涨跌幅"] = pd.to_numeric(
        temp_df["上证指数-涨跌幅"], errors="coerce"
    )
    temp_df.sort_values(["数据日期"], ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    stock_account_statistics_em_df = stock_account_statistics_em()
    print(stock_account_statistics_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/8/18 11:00
Desc: 大连商品交易所-业务/服务-业务参数-交易参数-合约信息查询
http://www.dce.com.cn/dalianshangpin/ywfw/ywcs/jycs/hyxxcx/index.html
"""

import pandas as pd
import requests


def futures_contract_info_dce() -> pd.DataFrame:
    """
    大连商品交易所-数据中心-业务数据-交易参数-合约信息
    http://www.dce.com.cn/dce/channel/list/180.html
    :return: 交易参数汇总查询
    :rtype: pandas.DataFrame
    """
    url = "http://www.dce.com.cn/dcereport/publicweb/tradepara/contractInfo"
    payload = {
        "lang": "zh",
        "tradeType": "1",
        "varietyId": "all",
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data'])
    temp_df.rename(columns={
        "contractId": "合约",
        "variety": "品种名称",
        "varietyOrder": "品种代码",
        "unit": "交易单位",
        "tick": "最小变动价位",
        "startTradeDate": "开始交易日",
        "endTradeDate": "最后交易日",
        "endDeliveryDate": "最后交割日",
        "tradeType": "",
    }, inplace=True)
    temp_df = temp_df[[
        "品种名称",
        "合约",
        "交易单位",
        "最小变动价位",
        "开始交易日",
        "最后交易日",
        "最后交割日",
    ]]
    temp_df["交易单位"] = pd.to_numeric(temp_df["交易单位"], errors="coerce")
    temp_df["最小变动价位"] = pd.to_numeric(temp_df["最小变动价位"], errors="coerce")
    temp_df["开始交易日"] = pd.to_datetime(
        temp_df["开始交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["最后交易日"] = pd.to_datetime(
        temp_df["最后交易日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["最后交割日"] = pd.to_datetime(
        temp_df["最后交割日"], format="%Y%m%d", errors="coerce"
    ).dt.date
    return temp_df


if __name__ == "__main__":
    futures_contract_info_dce_df = futures_contract_info_dce()
    print(futures_contract_info_dce_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/4/5 17:30
Desc: 东方财富网-行情中心-债券市场-质押式回购
https://quote.eastmoney.com/center/gridlist.html#bond_sz_buyback
"""

import pandas as pd

import requests


def bond_sh_buy_back_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-债券市场-上证质押式回购
    https://quote.eastmoney.com/center/gridlist.html#bond_sh_buyback
    :return: 上证质押式回购
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "np": "1",
        "fltt": "1",
        "invt": "2",
        "fs": "m:1+b:MK0356",
        "fields": "f12,f13,f14,f1,f2,f4,f3,f152,f17,f18,f15,f16,f5,f6",
        "fid": "f6",
        "pn": "1",
        "pz": "20",
        "po": "1",
        "dect": "1",
        "wbp2u": "|0|0|0|web",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data']['diff'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = temp_df['index'] + 1
    temp_df.rename(
        columns={
            "index": "序号",
            "f2": "最新价",
            "f3": "涨跌幅",
            "f4": "涨跌额",
            "f5": "成交量",
            "f6": "成交额",
            "f12": "代码",
            "f14": "名称",
            "f15": "最高",
            "f16": "最低",
            "f17": "今开",
            "f18": "昨收",
        },
        inplace=True,
    )

    temp_df = temp_df[[
        "序号",
        "代码",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "今开",
        "最高",
        "最低",
        "昨收",
        "成交量",
        "成交额",
    ]]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce") / 1000
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce") / 100
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce") / 1000
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce") / 1000
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce") / 1000
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce") / 1000
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce") / 1000
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


def bond_sz_buy_back_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-债券市场-深证质押式回购
    https://quote.eastmoney.com/center/gridlist.html#bond_sz_buyback
    :return: 深证质押式回购
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "np": "1",
        "fltt": "1",
        "invt": "2",
        "fs": "m:0+b:MK0356",
        "fields": "f12,f13,f14,f1,f2,f4,f3,f152,f17,f18,f15,f16,f5,f6",
        "fid": "f6",
        "pn": "1",
        "pz": "20",
        "po": "1",
        "dect": "1",
        "wbp2u": "|0|0|0|web",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data']['diff'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = temp_df['index'] + 1
    temp_df.rename(
        columns={
            "index": "序号",
            "f2": "最新价",
            "f3": "涨跌幅",
            "f4": "涨跌额",
            "f5": "成交量",
            "f6": "成交额",
            "f12": "代码",
            "f14": "名称",
            "f15": "最高",
            "f16": "最低",
            "f17": "今开",
            "f18": "昨收",
        },
        inplace=True,
    )

    temp_df = temp_df[[
        "序号",
        "代码",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "今开",
        "最高",
        "最低",
        "昨收",
        "成交量",
        "成交额",
    ]]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce") / 1000
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce") / 100
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce") / 1000
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce") / 1000
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce") / 1000
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce") / 1000
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce") / 1000
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


def bond_buy_back_hist_em(symbol: str = "204001"):
    """
    东方财富网-行情中心-债券市场-质押式回购-历史数据
    https://quote.eastmoney.com/center/gridlist.html#bond_sh_buyback
    :param symbol: 质押式回购代码
    :type symbol: str
    :return: 历史数据
    :rtype: pandas.DataFrame
    """
    if symbol.startswith("1"):
        market_id = "0"
    else:
        market_id = "1"
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "secid": f"{market_id}.{symbol}",
        "klt": "101",
        "fqt": "1",
        "lmt": "10000",
        "end": "20500000",
        "iscca": "1",
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64",
        "forcect": "1"
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(',') for item in data_json['data']['klines']])
    temp_df.columns = [
            "日期",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
    ]
    temp_df = temp_df[[
        "日期",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
    ]]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    bond_sh_buy_back_em_df = bond_sh_buy_back_em()
    print(bond_sh_buy_back_em_df)

    bond_sz_buy_back_em_df = bond_sz_buy_back_em()
    print(bond_sz_buy_back_em_df)

    bond_buy_back_hist_em_df = bond_buy_back_hist_em(symbol="204001")
    print(bond_buy_back_hist_em_df)

    bond_buy_back_hist_em_df = bond_buy_back_hist_em(symbol="131810")
    print(bond_buy_back_hist_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/22 18:00
Desc: 东方财富网-数据中心-资金流向-排名
https://data.eastmoney.com/zjlx/detail.html
异步接口-测试版
"""

import asyncio
from typing import Dict, List

import aiohttp
import pandas as pd


async def fetch_single_page(
    session: aiohttp.ClientSession, url: str, params: Dict
) -> Dict:
    """异步获取单页数据"""
    async with session.get(url, params=params, ssl=False) as response:
        return await response.json()


async def fetch_all_pages_async(url: str, base_params: Dict) -> List[Dict]:
    """异步获取所有页面数据"""
    # 首先获取总数以计算页数
    first_page_params = base_params.copy()
    first_page_params["pn"] = "1"

    async with aiohttp.ClientSession() as session:
        first_page_data = await fetch_single_page(session, url, first_page_params)

        # 检查是否成功获取数据
        if first_page_data.get("rc") != 0 or not first_page_data.get("data"):
            return [first_page_data]  # 返回错误信息

        total = first_page_data["data"]["total"]
        page_size = int(base_params["pz"])
        total_pages = (total + page_size - 1) // page_size

        # 限制页数，避免过大请求
        total_pages = min(total_pages, 100)

        # 创建所有页面的任务
        tasks = []
        for page in range(1, total_pages + 1):
            page_params = base_params.copy()
            page_params["pn"] = str(page)
            tasks.append(fetch_single_page(session, url, page_params))

        # 并发执行所有任务
        results = await asyncio.gather(*tasks)
        return results


def process_fund_flow_data(page_results: List[Dict], indicator: str) -> pd.DataFrame:
    """处理资金流向排名数据，转换为DataFrame"""
    all_data = []

    for result in page_results:
        if result.get("data") and result["data"].get("diff"):
            page_data = result["data"]["diff"]
            all_data.extend(page_data)

    if not all_data:
        return pd.DataFrame()

    temp_df = pd.DataFrame(all_data)

    # 根据不同的指标设置列名并选择需要的列
    if indicator == "今日":
        columns_map = {
            "f12": "代码",
            "f14": "名称",
            "f2": "最新价",
            "f3": "今日涨跌幅",
            "f62": "今日主力净流入-净额",
            "f184": "今日主力净流入-净占比",
            "f66": "今日超大单净流入-净额",
            "f69": "今日超大单净流入-净占比",
            "f72": "今日大单净流入-净额",
            "f75": "今日大单净流入-净占比",
            "f78": "今日中单净流入-净额",
            "f81": "今日中单净流入-净占比",
            "f84": "今日小单净流入-净额",
            "f87": "今日小单净流入-净占比",
        }
        main_column = "f62"  # 今日主力净流入-净额

    elif indicator == "3日":
        columns_map = {
            "f12": "代码",
            "f14": "名称",
            "f2": "最新价",
            "f127": "3日涨跌幅",
            "f267": "3日主力净流入-净额",
            "f268": "3日主力净流入-净占比",
            "f269": "3日超大单净流入-净额",
            "f270": "3日超大单净流入-净占比",
            "f271": "3日大单净流入-净额",
            "f272": "3日大单净流入-净占比",
            "f273": "3日中单净流入-净额",
            "f274": "3日中单净流入-净占比",
            "f275": "3日小单净流入-净额",
            "f276": "3日小单净流入-净占比",
        }
        main_column = "f267"  # 3日主力净流入-净额

    elif indicator == "5日":
        columns_map = {
            "f12": "代码",
            "f14": "名称",
            "f2": "最新价",
            "f109": "5日涨跌幅",
            "f164": "5日主力净流入-净额",
            "f165": "5日主力净流入-净占比",
            "f166": "5日超大单净流入-净额",
            "f167": "5日超大单净流入-净占比",
            "f168": "5日大单净流入-净额",
            "f169": "5日大单净流入-净占比",
            "f170": "5日中单净流入-净额",
            "f171": "5日中单净流入-净占比",
            "f172": "5日小单净流入-净额",
            "f173": "5日小单净流入-净占比",
        }
        main_column = "f164"  # 5日主力净流入-净额

    elif indicator == "10日":
        columns_map = {
            "f12": "代码",
            "f14": "名称",
            "f2": "最新价",
            "f160": "10日涨跌幅",
            "f174": "10日主力净流入-净额",
            "f175": "10日主力净流入-净占比",
            "f176": "10日超大单净流入-净额",
            "f177": "10日超大单净流入-净占比",
            "f178": "10日大单净流入-净额",
            "f179": "10日大单净流入-净占比",
            "f180": "10日中单净流入-净额",
            "f181": "10日中单净流入-净占比",
            "f182": "10日小单净流入-净额",
            "f183": "10日小单净流入-净占比",
        }
        main_column = "f174"  # 10日主力净流入-净额

    # 确保数值型列为数值类型
    numeric_columns = [
        col
        for col in temp_df.columns
        if col.startswith("f") and col not in ["f12", "f14"]
    ]
    for col in numeric_columns:
        if col in temp_df.columns:
            temp_df[col] = pd.to_numeric(temp_df[col], errors="coerce")

    # 按照主力净流入额降序排序
    if main_column in temp_df.columns:
        temp_df.sort_values(by=main_column, ascending=False, inplace=True)

    # 首先重命名列
    temp_df.rename(columns=columns_map, inplace=True)

    # 选择需要的列
    selected_columns = list(columns_map.values())
    available_columns = [col for col in selected_columns if col in temp_df.columns]
    temp_df = temp_df[available_columns]

    # 重置索引并生成序号列
    temp_df.reset_index(drop=True, inplace=True)
    temp_df.insert(0, "序号", range(1, len(temp_df) + 1))

    return temp_df


async def stock_individual_fund_flow_rank_async(indicator: str = "5日") -> pd.DataFrame:
    """
    异步获取东方财富网-数据中心-资金流向-排名
    https://data.eastmoney.com/zjlx/detail.html
    :param indicator: choice of {"今日", "3日", "5日", "10日"}
    :type indicator: str
    :return: 指定 indicator 资金流向排行
    :rtype: pandas.DataFrame
    """
    indicator_map = {
        "今日": [
            "f62",
            "f12,f14,f2,f3,f62,f184,f66,f69,f72,f75,f78,f81,f84,f87,f204,f205,f124",
        ],
        "3日": [
            "f267",
            "f12,f14,f2,f127,f267,f268,f269,f270,f271,f272,f273,f274,f275,f276,f257,f258,f124",
        ],
        "5日": [
            "f164",
            "f12,f14,f2,f109,f164,f165,f166,f167,f168,f169,f170,f171,f172,f173,f257,f258,f124",
        ],
        "10日": [
            "f174",
            "f12,f14,f2,f160,f174,f175,f176,f177,f178,f179,f180,f181,f182,f183,f260,f261,f124",
        ],
    }

    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "fid": indicator_map[indicator][0],
        "po": "1",  # 按照降序排列
        "pz": "100",  # 每页100条
        "pn": "1",  # 从第1页开始
        "np": "1",
        "fltt": "2",
        "invt": "2",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
        "fs": "m:0+t:6+f:!2,m:0+t:13+f:!2,m:0+t:80+f:!2,m:1+t:2+f:!2,m:1+t:23+f:!2,m:0+t:7+f:!2,m:1+t:3+f:!2",
        "fields": indicator_map[indicator][1],
    }

    results = await fetch_all_pages_async(url, params)
    return process_fund_flow_data(results, indicator)


def stock_individual_fund_flow_rank(indicator: str = "5日") -> pd.DataFrame:
    """
    东方财富网-数据中心-资金流向-排名 (同步接口)
    https://data.eastmoney.com/zjlx/detail.html
    :param indicator: choice of {"今日", "3日", "5日", "10日"}
    :type indicator: str
    :return: 指定 indicator 资金流向排行
    :rtype: pandas.DataFrame
    """
    import nest_asyncio

    nest_asyncio.apply()
    return asyncio.run(stock_individual_fund_flow_rank_async(indicator))


if __name__ == "__main__":
    # 测试同步接口
    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="今日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="3日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="5日"
    )
    print(stock_individual_fund_flow_rank_df)

    stock_individual_fund_flow_rank_df = stock_individual_fund_flow_rank(
        indicator="10日"
    )
    print(stock_individual_fund_flow_rank_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/7 19:30
Desc: 限售股解禁
https://data.eastmoney.com/dxf/detail.html
"""

import pandas as pd
import requests
from tqdm import tqdm


def stock_restricted_release_summary_em(
    symbol: str = "全部股票", start_date: str = "20221101", end_date: str = "20221209"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-限售股解禁
    https://data.eastmoney.com/dxf/marketStatistics.html?type=day&startdate=2022-11-08&enddate=2022-12-19
    :param symbol: 标的市场; choice of {"全部股票", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"}
    :type symbol: str
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :return: 限售股解禁
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部股票": "000300",
        "沪市A股": "000001",
        "科创板": "000688",
        "深市A股": "399001",
        "创业板": "399001",
        "京市A股": "999999",
    }
    start_date_str = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date_str = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "FREE_DATE",
        "sortTypes": "1",
        "pageSize": "500",
        "pageNumber": "1",
        "columns": "ALL",
        "quoteColumns": "f2~03~INDEX_CODE,f3~03~INDEX_CODE,f124~03~INDEX_CODE",
        "quoteType": "0",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(INDEX_CODE="{symbol_map[symbol]}")(FREE_DATE>=
        '{start_date_str}')(FREE_DATE<='{end_date_str}')""",
        "reportName": "RPT_LIFTDAY_STA",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"] + 1
    temp_df.columns = [
        "序号",
        "解禁时间",
        "当日解禁股票家数",
        "实际解禁数量",
        "实际解禁市值",
        "沪深300指数",
        "沪深300指数涨跌幅",
        "解禁数量",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "序号",
            "解禁时间",
            "当日解禁股票家数",
            "解禁数量",
            "实际解禁数量",
            "实际解禁市值",
            "沪深300指数",
            "沪深300指数涨跌幅",
        ]
    ]
    temp_df["解禁时间"] = pd.to_datetime(temp_df["解禁时间"], errors="coerce").dt.date
    temp_df["当日解禁股票家数"] = pd.to_numeric(
        temp_df["当日解禁股票家数"], errors="coerce"
    )
    temp_df["解禁数量"] = pd.to_numeric(temp_df["解禁数量"], errors="coerce") * 10000
    temp_df["实际解禁数量"] = (
        pd.to_numeric(temp_df["实际解禁数量"], errors="coerce") * 10000
    )
    temp_df["实际解禁市值"] = (
        pd.to_numeric(temp_df["实际解禁市值"], errors="coerce") * 10000
    )
    temp_df["沪深300指数"] = pd.to_numeric(temp_df["沪深300指数"], errors="coerce")
    temp_df["沪深300指数涨跌幅"] = pd.to_numeric(
        temp_df["沪深300指数涨跌幅"], errors="coerce"
    )
    return temp_df


def stock_restricted_release_detail_em(
    start_date: str = "20221202", end_date: str = "20241202"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-限售股解禁-解禁详情一览
    https://data.eastmoney.com/dxf/detail.html
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :return: 解禁详情一览
    :rtype: pandas.DataFrame
    """
    start_date_str = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date_str = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "FREE_DATE,CURRENT_FREE_SHARES",
        "sortTypes": "1,1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_LIFT_STAGE",
        "columns": "SECURITY_CODE,SECURITY_NAME_ABBR,FREE_DATE,CURRENT_FREE_SHARES,ABLE_FREE_SHARES,"
                   "LIFT_MARKET_CAP,FREE_RATIO,NEW,B20_ADJCHRATE,A20_ADJCHRATE,FREE_SHARES_TYPE,TOTAL_RATIO,"
                   "NON_FREE_SHARES,BATCH_HOLDER_NUM",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(FREE_DATE>='{start_date_str}')(FREE_DATE<='{end_date_str}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df["index"] + 1
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "解禁时间",
        "实际解禁数量",
        "解禁数量",
        "实际解禁市值",
        "占解禁前流通市值比例",
        "解禁前一交易日收盘价",
        "解禁前20日涨跌幅",
        "解禁后20日涨跌幅",
        "限售股类型",
        "-",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "解禁时间",
            "限售股类型",
            "解禁数量",
            "实际解禁数量",
            "实际解禁市值",
            "占解禁前流通市值比例",
            "解禁前一交易日收盘价",
            "解禁前20日涨跌幅",
            "解禁后20日涨跌幅",
        ]
    ]
    big_df["解禁时间"] = pd.to_datetime(big_df["解禁时间"], errors="coerce").dt.date

    big_df["解禁数量"] = pd.to_numeric(big_df["解禁数量"], errors="coerce") * 10000
    big_df["实际解禁数量"] = (
        pd.to_numeric(big_df["实际解禁数量"], errors="coerce") * 10000
    )
    big_df["实际解禁市值"] = (
        pd.to_numeric(big_df["实际解禁市值"], errors="coerce") * 10000
    )
    big_df["占解禁前流通市值比例"] = pd.to_numeric(
        big_df["占解禁前流通市值比例"], errors="coerce"
    )
    big_df["解禁前一交易日收盘价"] = pd.to_numeric(
        big_df["解禁前一交易日收盘价"], errors="coerce"
    )
    big_df["解禁前20日涨跌幅"] = pd.to_numeric(
        big_df["解禁前20日涨跌幅"], errors="coerce"
    )
    big_df["解禁后20日涨跌幅"] = pd.to_numeric(
        big_df["解禁后20日涨跌幅"], errors="coerce"
    )
    return big_df


def stock_restricted_release_queue_em(symbol: str = "600000") -> pd.DataFrame:
    """
    东方财富网-数据中心-个股限售解禁-解禁批次
    https://data.eastmoney.com/dxf/q/600000.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 个股限售解禁
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "FREE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_LIFT_STAGE",
        "filter": f'(SECURITY_CODE="{symbol}")',
        "columns": "SECURITY_CODE,SECURITY_NAME_ABBR,FREE_DATE,CURRENT_FREE_SHARES,ABLE_FREE_SHARES,"
                   "LIFT_MARKET_CAP,FREE_RATIO,NEW,B20_ADJCHRATE,A20_ADJCHRATE,FREE_SHARES_TYPE,TOTAL_RATIO,"
                   "NON_FREE_SHARES,BATCH_HOLDER_NUM",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["result"]:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"] + 1
    temp_df.columns = [
        "序号",
        "-",
        "-",
        "解禁时间",
        "实际解禁数量",
        "解禁数量",
        "实际解禁数量市值",
        "占流通市值比例",
        "解禁前一交易日收盘价",
        "解禁前20日涨跌幅",
        "解禁后20日涨跌幅",
        "限售股类型",
        "占总市值比例",
        "未解禁数量",
        "解禁股东数",
    ]
    temp_df = temp_df[
        [
            "序号",
            "解禁时间",
            "解禁股东数",
            "解禁数量",
            "实际解禁数量",
            "未解禁数量",
            "实际解禁数量市值",
            "占总市值比例",
            "占流通市值比例",
            "解禁前一交易日收盘价",
            "限售股类型",
            "解禁前20日涨跌幅",
            "解禁后20日涨跌幅",
        ]
    ]
    temp_df["解禁时间"] = pd.to_datetime(temp_df["解禁时间"], errors="coerce").dt.date
    temp_df["解禁股东数"] = pd.to_numeric(temp_df["解禁股东数"], errors="coerce")
    temp_df["解禁数量"] = pd.to_numeric(temp_df["解禁数量"], errors="coerce") * 10000
    temp_df["实际解禁数量"] = (
        pd.to_numeric(temp_df["实际解禁数量"], errors="coerce") * 10000
    )
    temp_df["未解禁数量"] = (
        pd.to_numeric(temp_df["未解禁数量"], errors="coerce") * 10000
    )
    temp_df["实际解禁数量市值"] = (
        pd.to_numeric(temp_df["实际解禁数量市值"], errors="coerce") * 10000
    )
    temp_df["占总市值比例"] = pd.to_numeric(temp_df["占总市值比例"], errors="coerce")
    temp_df["占流通市值比例"] = pd.to_numeric(
        temp_df["占流通市值比例"], errors="coerce"
    )
    temp_df["解禁前一交易日收盘价"] = pd.to_numeric(
        temp_df["解禁前一交易日收盘价"], errors="coerce"
    )
    temp_df["解禁前20日涨跌幅"] = pd.to_numeric(
        temp_df["解禁前20日涨跌幅"], errors="coerce"
    )
    temp_df["解禁后20日涨跌幅"] = pd.to_numeric(
        temp_df["解禁后20日涨跌幅"], errors="coerce"
    )
    return temp_df


def stock_restricted_release_stockholder_em(
    symbol: str = "600000", date: str = "20200904"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-个股限售解禁-解禁股东
    https://data.eastmoney.com/dxf/q/600000.html
    :param symbol: 股票代码
    :type symbol: str
    :param date: 日期; 通过 ak.stock_restricted_release_queue_em(symbol="600000") 获取
    :type date: str
    :return: 个股限售解禁
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    date_str = "-".join([date[:4], date[4:6], date[6:]])
    params = {
        "sortColumns": "ADD_LISTING_SHARES",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_LIFT_GD",
        "filter": f"""(SECURITY_CODE="{symbol}")(FREE_DATE='{date_str}')""",
        "columns": "LIMITED_HOLDER_NAME,ADD_LISTING_SHARES,ACTUAL_LISTED_SHARES,ADD_LISTING_CAP,LOCK_MONTH,"
                   "RESIDUAL_LIMITED_SHARES,FREE_SHARES_TYPE,PLAN_FEATURE",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"] + 1
    temp_df.columns = [
        "序号",
        "股东名称",
        "解禁数量",
        "实际解禁数量",
        "解禁市值",
        "锁定期",
        "剩余未解禁数量",
        "限售股类型",
        "进度",
    ]
    temp_df["解禁数量"] = pd.to_numeric(temp_df["解禁数量"], errors="coerce")
    temp_df["实际解禁数量"] = pd.to_numeric(temp_df["实际解禁数量"], errors="coerce")
    temp_df["解禁市值"] = pd.to_numeric(temp_df["解禁市值"], errors="coerce")
    temp_df["锁定期"] = pd.to_numeric(temp_df["锁定期"], errors="coerce")
    temp_df["剩余未解禁数量"] = pd.to_numeric(
        temp_df["剩余未解禁数量"], errors="coerce"
    )
    temp_df["剩余未解禁数量"] = pd.to_numeric(
        temp_df["剩余未解禁数量"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    stock_restricted_release_summary_em_df = stock_restricted_release_summary_em(
        symbol="全部股票", start_date="20221108", end_date="20221209"
    )
    print(stock_restricted_release_summary_em_df)

    stock_restricted_release_detail_em_df = stock_restricted_release_detail_em(
        start_date="20221202", end_date="20221204"
    )
    print(stock_restricted_release_detail_em_df)

    stock_restricted_release_queue_em_df = stock_restricted_release_queue_em(
        symbol="600000"
    )
    print(stock_restricted_release_queue_em_df)

    stock_restricted_release_stockholder_em_df = (
        stock_restricted_release_stockholder_em(symbol="600000", date="20200904")
    )
    print(stock_restricted_release_stockholder_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/4/10 19:58
Desc: Economic Research from Federal Reserve Bank of St. Louis
https://research.stlouisfed.org/econ/mccracken/fred-databases/
FRED-MD and FRED-QD are large macroeconomic databases designed for the empirical analysis of “big data.” The datasets of monthly and quarterly observations mimic the coverage of datasets already used in the literature, but they add three appealing features. They are updated in real-time through the FRED database. They are publicly accessible, facilitating the replication of empirical work. And they relieve the researcher of the task of incorporating data changes and revisions (a task accomplished by the data desk at the Federal Reserve Bank of St. Louis).
"""

import pandas as pd


def fred_md(date: str = "2020-01") -> pd.DataFrame:
    """
    The accompanying paper shows that factors extracted from the FRED-MD dataset share the same predictive content as those based on the various vintages of the so-called Stock-Watson data. In addition, it suggests that diffusion indexes constructed as the partial sum of the factor estimates can potentially be useful for the study of business cycle chronology.
    :param date: e.g., "2020-03"; from "2015-01" to now
    :type date: str
    :return: Monthly Data
    :rtype: pandas.DataFrame
    """
    url = (
        f"https://s3.amazonaws.com/files.fred.stlouisfed.org/fred-md/monthly/{date}.csv"
    )
    temp_df = pd.read_csv(url)
    return temp_df


def fred_qd(date: str = "2020-01") -> pd.DataFrame:
    """
    FRED-QD is a quarterly frequency companion to FRED-MD. It is designed to emulate the dataset used in "Disentangling the Channels of the 2007-2009 Recession" by Stock and Watson (2012, NBER WP No. 18094) but also contains several additional series. Comments or suggestions are welcome.
    :param date: e.g., "2020-03"; from "2015-01" to now
    :type date: str
    :return: Quarterly Data
    :rtype: pandas.DataFrame
    """
    url = f"https://s3.amazonaws.com/files.fred.stlouisfed.org/fred-md/quarterly/{date}.csv"
    temp_df = pd.read_csv(url)
    return temp_df


if __name__ == "__main__":
    fred_md_df = fred_md(date="2023-03")
    print(fred_md_df)

    fred_qd_df = fred_qd(date="2023-03")
    print(fred_qd_df)
